<?php
namespace Backend\Model;
use Backend\Model\BackendModel;

/**
 * -------------------------------------------------------
 * | 产品模型类
 * | @author: duanbin
 * | @时间: 2016年10月9日 下午3:34:41
 * | @version: 1.0
 * -------------------------------------------------------
 */
class ProductPriceModel extends BackendModel{
	
	//关联的表名
	protected $trueTableName = 'agent_product_price';
	//新增是允许写入的字段
	protected $insertFields = [
			'product_id', 'month', 'product_price',
			'type', 'note_appended', 'up_time',
			'state', 'giant_price', 'min_price', 
			'create_time', 'area_code', 'api_type'
	];
	//更新是允许更新的字段
	protected $updateFields = [
			'month', 'product_price', 'type',
			'note_appended', 'up_time', 'state',
	];
	
	
	
	//搜索条件字段
	public $searchable = [
			'area_code' => [
					'display_name' => '所属区域',
					'html_type' => 'select',
					'data' => [],
			],
			'state' => [
					'display_name' => '状态',
					'html_type' => 'select',
					'data' => [
							'0' => '无效',
							'1' => '有效',
					],
			],
			'type' => [
					'display_name' => '类型',
					'html_type' => 'select',
					'data' => [
							'0' => '订购',
							'1' => '续费',
					],
			],
			'up_time' => [
					'display_name' => '更新时间',
					'html_type' => 'date',
					'start_name' => '_start',
					'end_name' => '_end',
			],
			'key' => [
				'note_appended' => '价格描述',
			],
	];
	
	
	//排序字段
	public $sortable = [
			'id' => [
					'display_name' =>'编号',
					'sortable' => true,
			],
			'product_name' => [
					'display_name' =>'名称',
					'sortable' => false,
			],
			'month' => [
					'display_name' =>'购买期限',
					'sortable' => false,
			],
			'giant_price' => [
					'display_name' => '景安价格',
					'sortable' => false,
			],
			'product_price' => [
					'display_name' => '产品价格',
					'sortable' => false,
			],
			'min_price' => [
					'display_name' => '最低价格',
					'sortable' => true,
			],
			'type' => [
					'display_name' => '类型',
					'sortable' => false,
			],
			'note_appended' => [
					'display_name' => '价格描述',
					'sortable' => false,		
			],
			'state' => [
					'display_name' => '状态',
					'sortable' => false,
			],
			'area_code' => [
					'display_name' => '地区',
					'sortable' => true,
			],
			'up_time' => [
					'display_name' => '修改时间',
					'sortable' => true,
			],
	];

	
	/**
	 * ----------------------------------------------
	 * | 获取搜索条件所需要的数据
	 * | 可以实现getFillData方法，
	 * | 将一些需要的数据复给$this->fill_data
	 * | 例如本类的getFillData()方法
	 * | @时间: 2016年10月11日 下午3:03:09
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	public function getSearchable($fill_data = []){
		if (empty($fill_data)) {
			return $this->fillSearchable($this->fillData);
		}else {
			return $this->fillSearchable($fill_data);
		}
	}
	
	
	/**
	 * ----------------------------------------------
	 * | 获取搜索条件需要的数据
	 * | @时间: 2016年10月12日 下午2:54:05
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: array
	 * ----------------------------------------------
	 */
	public function getFillData(){
		
		/**
		 * ---------------------------------------------------
		 * | 这里注意返回的一定是一个数组，
		 * | 并且数组的key一定是与searchable数组的key相同 
		 * | @时间: 2016年10月12日 下午2:54:52
		 * ---------------------------------------------------
		 */
		
		//产品分类数据
// 		$fill_data['product_type_id'] = D('Common/ProductType')->getAll('id as k,type_name as v');
		//区域数据
		$fill_data['area_code'] = D('Common/Region')->getAll('region_code as k,region_name as v');
		
		//父类里继承来的属性
		$this->fillData = $fill_data;
		return $this;
	}
	

	/**********************************新增更新回调/开始***********************************/
	
	/**
	 * ----------------------------------------------
	 * | 插入数据之前的回调
	 * | @时间: 2016年10月17日 上午11:28:21
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	protected function _before_insert(&$data, $options){
		
	}

	/**
	 * ----------------------------------------------
	 * | 插入成功后的回调
	 * | @时间: 2016年10月17日 上午11:28:50
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	protected function _after_insert($data, $options){
		
	}
	
	/**
	 * ----------------------------------------------
	 * | 更新数据之前做的操作
	 * | @时间: 2016年10月17日 上午11:26:36
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	protected function _before_update(&$data, $options){
		//查看产品价格是否合法
		if ( getDecimalDigit($data['product_price']) > 2 ){
			$this->error = '产品价格最多只有小数点后两位';
			return false;
		}
		//查看购买时间是否合法
		if ( !is_int($data['month']) ) {
			$this->error = '购买时间只能为整数';
			return false;			
		}
		//查看购买类型是否非法
		if ( !in_array($data['type'], [ 0,1 ], true) ) {
			$this->error = '购买类型参数非法';
			return false;						
		}
		//处理下价格说明的编码，统一转换成utf-8的编码，然后查看字数
		$note_appended = mb_convert_encoding($data['note_appended'], 'utf-8');
		if ( mb_strlen($note_appended, 'utf-8') >= 50*3) {
			$this->error = '价格说明最多只有50个字';
			return false;								
		}
		
		//添加更新时间默认值
		$data['up_time'] = date( 'Y-m-d H:i:s', time() );

	}

	/**
	 * ----------------------------------------------
	 * | 更新之后做的操作
	 * | @时间: 2016年10月17日 上午11:27:09
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	protected function _after_update($data, $options){
		
	}
	
	/**********************************新增更新回调/结束***********************************/
	
	/**
	 * ----------------------------------------------
	 * | 批量修改价格
	 * | @时间: 2016年10月21日 上午11:53:53
	 * | @author: duanbin
	 * | @param: variable
	 * | @return: type
	 * ----------------------------------------------
	 */
	public function updateBatchPrice(){
		$request = request();
		
// 		dump($request);die;
		$m_product_price = M('product_price');
		$where['month'] = $request['month'];
		$updateField = 'product_price';
		$res = [];
		$ok = $error = 0;
		
		foreach ( $request['product_ids'] as $k => $v ){
			$where['product_id'] = [ 'eq', $v ];
			$product_price = $m_product_price->where($where);
			$product_price_info = $product_price->find();
			//1代表+
			if ($request['operator'] == 1) {
				$res[$v] = $m_product_price->where($where)->setInc($updateField, $request['value']);
				//2代表-
			}elseif ($request['operator'] == 2) {
				$price = $product_price_info["product_price"] - $request['value'];
                if($product_price_info["min_price"] < $price){
					$price = $product_price_info["min_price"];
                }
				$res[$v] = $m_product_price->where($where)->setField($updateField,$price);
				//3代表*
			}elseif ($request['operator'] == 3) {
				//这里要把价钱就代表折扣，要除以10
                $price = intval($product_price_info["giant_price"]*($request['value']/10));
                if($product_price_info["min_price"] < $price){
					$price = $product_price_info["min_price"];
                }
				$res[$v] = $m_product_price->where($where)->setField($updateField, $price);
			}
			//统计成功错误
			if ($res[$v] === false) {
				$error++;
			}else {
				$ok++;
			}
		}
		$this->error = '共修改成功'.$ok.'个:'.'修改失败'.$error.'个';
		return $res;
	}
}
